package com.gemserk.commons.animation.handlers;

import java.util.HashMap;
import java.util.Map;

import com.gemserk.commons.animation.Animation;
import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;

public class AnimationHandlerManager {

	Map<String, AnimationEntry> animations = new HashMap<String, AnimationEntry>();

	Multimap<String, AnimationEventHandler> animationEventHandlers = HashMultimap.create();

	class AnimationEntry {

		boolean wasFinished = true;

		boolean wasStarted = false;

		Animation animation;

		public AnimationEntry(Animation animation) {
			this.animation = animation;
		}

	}

	public void addAnimationHandler(String animationId, AnimationEventHandler animationEventHandler) {
		animationEventHandlers.put(animationId, animationEventHandler);
	}

	public void checkAnimationChanges() {
		for (String animationId : animationEventHandlers.keySet()) {
			AnimationEntry animationEntry = animations.get(animationId);
			Animation animation = animationEntry.animation;

			boolean callOnStart = animation.isStarted() && !animationEntry.wasStarted && !animation.isFinished();
			boolean callOnFinish = animation.isFinished() && !animationEntry.wasFinished;

			animationEntry.wasStarted = animation.isStarted();
			animationEntry.wasFinished = animation.isFinished();

			if (!callOnFinish && !callOnStart)
				continue;

			for (AnimationEventHandler animationEventHandler : animationEventHandlers.get(animationId)) {
				if (callOnFinish)
					animationEventHandler.onAnimationFinished(animation);
				if (callOnStart)
					animationEventHandler.onAnimationStarted(animation);
			}

		}
	}

	public void handleAnimationChanges(String animationId, Animation animation) {
		animations.put(animationId, new AnimationEntry(animation));
	}

	public Animation getAnimation(String animationId) {
		AnimationEntry animationEntry = animations.get(animationId);
		if (animationEntry != null)
			return animationEntry.animation;
		return null;
	}

}