package com.gemserk.commons.animation.timeline;

import java.util.Comparator;
import java.util.LinkedList;

public class TimelineValue<T> {

	@SuppressWarnings("unchecked")
	static class KeyFrameComparator implements Comparator<TimelineValue.KeyFrame> {
		@Override
		public int compare(TimelineValue.KeyFrame o1, TimelineValue.KeyFrame o2) {
			return (int) (o1.time - o2.time);
		}
	}

	static class KeyFrame<T> {

		float time;

		T value;

		public float getTime() {
			return time;
		}

		public T getValue() {
			return value;
		}

		public KeyFrame(float time, T value) {
			this.time = time;
			this.value = value;
		}

	}

	LinkedList<TimelineValue.KeyFrame<T>> keyFrames = new LinkedList<TimelineValue.KeyFrame<T>>();

	Interpolator<T> interpolator;

	public void setInterpolator(Interpolator<T> interpolator) {
		this.interpolator = interpolator;
	}

	public Interpolator<T> getInterpolator() {
		return interpolator;
	}

	public void addKeyFrame(float time, T value) {
		keyFrames.add(new TimelineValue.KeyFrame<T>(time, value));
	}

	public T getValue(float time) {

		KeyFrame<T> firstKeyFrame = keyFrames.getFirst();

		if (time <= firstKeyFrame.time)
			return firstKeyFrame.getValue();

		for (int i = 0; i < keyFrames.size(); i++) {

			TimelineValue.KeyFrame<T> currentKeyFrame = keyFrames.get(i);

			if (currentKeyFrame.time > time) {

				TimelineValue.KeyFrame<T> previousKeyFrame = keyFrames.get(i - 1);

				if (previousKeyFrame == null)
					throw new RuntimeException("invalid time " + time);

				float interval = currentKeyFrame.getTime() - previousKeyFrame.getTime();

				float weight = (time - previousKeyFrame.getTime()) / interval;

				return interpolator.interpolate(previousKeyFrame.getValue(), currentKeyFrame.getValue(), weight);

			}

		}

		return keyFrames.getLast().getValue();
	}

}