package com.gemserk.games.preld19.entities;

import java.util.Map;

import org.newdawn.slick.geom.Rectangle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.gemserk.componentsengine.commons.components.ImageRenderableComponent;
import com.gemserk.componentsengine.game.GlobalProperties;
import com.gemserk.componentsengine.input.InputMappingBuilder;
import com.gemserk.componentsengine.input.InputMappingBuilderConfigurator;
import com.gemserk.componentsengine.input.KeyboardMappingBuilder;
import com.gemserk.componentsengine.messages.MessageQueue;
import com.gemserk.componentsengine.properties.FixedProperty;
import com.gemserk.componentsengine.properties.Properties;
import com.gemserk.componentsengine.slick.utils.SlickUtils;
import com.gemserk.componentsengine.templates.EntityBuilder;
import com.gemserk.resources.Resource;
import com.gemserk.resources.ResourceManager;
import com.google.inject.Inject;
import com.google.inject.Provider;

public class WorldEntityBuilder extends EntityBuilder {

	protected static final Logger logger = LoggerFactory.getLogger(WorldEntityBuilder.class);

	@Inject
	Provider<InputMappingBuilderConfigurator> inputMappingConfiguratorProvider;

	@Inject
	MessageQueue messageQueue;

	@Inject
	GlobalProperties globalProperties;

	@Inject
	SlickUtils slick;

	@Inject
	ResourceManager resourceManager;


	@Override
	public void build() {

		tags("world");

		final Rectangle screenBounds = (Rectangle) parameters.get("screenBounds");
		final Map<String, Object> level = (Map<String, Object>) parameters.get("level");

		property("bounds", screenBounds);
		property("points", parameters.get("points"));
		property("level", level);

		property("backgroundImageResource", resourceManager.get(level.get("background")));

		component(new ImageRenderableComponent("background")).withProperties(new ComponentProperties() {
			{
				// property("image", slick.getResources().image((String) level.get("background")));
				// property("image", resourceManager.get(level.get("background"), Image.class));
				property("color", slick.color(1, 1, 1, 1));
				property("position", slick.vector(screenBounds.getCenterX(), screenBounds.getCenterY()));
				property("direction", slick.vector(1, 0));
				property("layer", -1000);

				property("image", new FixedProperty(entity) {
					@Override
					public Object get() {
						Resource resource = Properties.getValue(getHolder(), "backgroundImageResource");
						return resource.get();
					}
				});
			}
		});


		component(inputMappingConfiguratorProvider.get().configure(new InputMappingBuilder("inputMappingComponent") {

			@Override
			public void build() {

				keyboard(new KeyboardMappingBuilder() {
					@Override
					public void build() {
					}
				});
			}

		}));
	}
}
