package com.gemserk.games.preld19.gamestates;

import java.util.HashMap;

import org.newdawn.slick.Music;
import org.newdawn.slick.Sound;
import org.newdawn.slick.geom.Rectangle;

import com.gemserk.componentsengine.commons.components.ImageRenderableComponent;
import com.gemserk.componentsengine.components.FieldsReflectionComponent;
import com.gemserk.componentsengine.components.annotations.EntityProperty;
import com.gemserk.componentsengine.components.annotations.Handles;
import com.gemserk.componentsengine.game.GlobalProperties;
import com.gemserk.componentsengine.input.InputMappingBuilder;
import com.gemserk.componentsengine.input.InputMappingBuilderConfigurator;
import com.gemserk.componentsengine.input.KeyboardMappingBuilder;
import com.gemserk.componentsengine.input.MouseMappingBuilder;
import com.gemserk.componentsengine.messages.Message;
import com.gemserk.componentsengine.messages.MessageQueue;
import com.gemserk.componentsengine.messages.messageBuilder.MessageBuilder;
import com.gemserk.componentsengine.properties.FixedProperty;
import com.gemserk.componentsengine.properties.Properties;
import com.gemserk.componentsengine.properties.PropertiesMapBuilder;
import com.gemserk.componentsengine.properties.ReferenceProperty;
import com.gemserk.componentsengine.slick.utils.SlickUtils;
import com.gemserk.componentsengine.templates.EntityBuilder;
import com.gemserk.componentsengine.templates.JavaEntityTemplate;
import com.gemserk.resources.Resource;
import com.gemserk.resources.ResourceManager;
import com.google.inject.Inject;
import com.google.inject.Provider;

public class MenuScreenEntityBuilder extends EntityBuilder {

	@Inject
	Provider<InputMappingBuilderConfigurator> inputMappingConfiguratorProvider;

	@Inject
	SlickUtils slick;

	@Inject
	GlobalProperties globalProperties;

	@Inject
	MessageQueue messageQueue;

	@Inject
	ResourceManager resourceManager;

	@Inject
	Provider<JavaEntityTemplate> javaEntityTemplateProvider;

	@Inject
	MessageBuilder messageBuilder;

	@Override
	public void build() {

		final Rectangle screenResolution = (Rectangle) globalProperties.getProperties().get("screenResolution");
		final Rectangle labelRectangle = slick.rectangle(-160, -25, 320, 50);

		property("fontResource", resourceManager.get("FontDialogMessage2"));
		property("font", new FixedProperty(entity) {
			@Override
			public Object get() {
				Resource fontResource = Properties.getValue(getHolder(), "fontResource");
				return fontResource.get();
			}
		});

		component(new ImageRenderableComponent("background")).withProperties(new ComponentProperties() {
			{
				property("position", slick.vector((float) (screenResolution.getWidth() * 0.5f), (float) (screenResolution.getHeight() * 0.5f)));
				property("color", slick.color(1, 1, 1, 1));
				property("direction", slick.vector(1, 0));
				property("layer", 0);
				property("image", new FixedProperty(entity) {
					@Override
					public Object get() {
						return resourceManager.get("background").get();
					}
				});
			}
		});

		child(templateProvider.getTemplate("gemserk.gui.label").instantiate("titleLabel", new HashMap<String, Object>() {
			{
				put("position", slick.vector(screenResolution.getCenterX(), 40f));
				put("color", slick.color(0.5f, 0.2f, 0.2f, 1f));
				put("bounds", labelRectangle);
				put("align", "center");
				put("valign", "center");
				put("layer", 1);
				put("message", "Main Menu");
				put("font", resourceManager.get("FontTitle2"));
					
			}
		}));

		// final InterpolatedPropertyTimeProvider timeProvider = new InterpolatedPropertyTimeProvider();
		//
		// component(new UpdateTimeProviderComponent("updateTimeProvider")).withProperties(new ComponentProperties() {
		// {
		// property("timeProvider", timeProvider);
		// }
		// });

		child(templateProvider.getTemplate("gui.button").instantiate("playButton", new HashMap<String, Object>() {
			{
				put("font", new ReferenceProperty<Object>("fontResource", entity));
				put("position", slick.vector(screenResolution.getCenterX(), screenResolution.getCenterY() - 50f));
				put("bounds", labelRectangle);
				put("align", "center");
				put("valign", "center");
				put("layer", 1);
				put("message", "Play");
			}
		}));

		child(templateProvider.getTemplate("gui.button").instantiate("settingsButton", new HashMap<String, Object>() {
			{
				put("font", new ReferenceProperty<Object>("fontResource", entity));
				put("position", slick.vector(screenResolution.getCenterX(), screenResolution.getCenterY()));
				put("bounds", labelRectangle);
				put("align", "center");
				put("valign", "center");
				put("layer", 1);
				put("message", "Settings");
			}
		}));

		child(templateProvider.getTemplate("gui.button").instantiate("exitButton", new HashMap<String, Object>() {
			{
				put("font", new ReferenceProperty<Object>("fontResource", entity));
				put("position", slick.vector(screenResolution.getCenterX(), screenResolution.getCenterY() + 50f));
				put("bounds", labelRectangle);
				put("align", "center");
				put("valign", "center");
				put("layer", 1);
				put("message", "Exit");
			}
		}));

		property("buttonPressed", "play");

		component(new FieldsReflectionComponent("guiHandler") {

			@EntityProperty
			String buttonPressed;

			@Handles
			public void buttonReleased(Message message) {
				String id = Properties.getValue(message, "buttonId");

				if ("playButton".equals(id)) {
					buttonPressed = "play";
				}

				if ("settingsButton".equals(id)) {
					buttonPressed = "settings";
				}

				if ("exitButton".equals(id)) {
					buttonPressed = "exit";
				}

				messageQueue.enqueue(messageBuilder.newMessage("handleGuiAction").property("action", buttonPressed).get());
			}

		}).withProperties(new ComponentProperties() {
			{
			}
		});

		component(new FieldsReflectionComponent("guiActionHandler") {


			@Handles
			public void handleGuiAction(Message message) {
				String buttonPressed = Properties.getValue(message, "action");
				if ("exit".equals(buttonPressed)) {
					System.exit(0);
				} else if ("settings".equals(buttonPressed)) {
					messageQueue.enqueue(new Message("settings"));
				} else if ("play".equals(buttonPressed)) {
					messageQueue.enqueue(new Message("resume"));
					messageQueue.enqueueDelay(new Message("restartLevel"));
				}

			}

		});

		component(inputMappingConfiguratorProvider.get().configure(new InputMappingBuilder("inputMappingComponent") {

			@Override
			public void build() {

				keyboard(new KeyboardMappingBuilder() {
					@Override
					public void build() {

					}
				});

				mouse(new MouseMappingBuilder() {
					@Override
					public void build() {

					}
				});
			}

		}));

	}
}